package ecp.vcs.com.ecpsyncplugin.filePicker;

import android.content.ContentUris;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.provider.DocumentsContract;
import android.provider.MediaStore;
import android.provider.OpenableColumns;
import android.view.View;
import android.widget.Toast;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import ecp.vcs.com.ecpsyncplugin.R;
import ecp.vcs.com.ecpsyncplugin.Utils;

public class FilePickerUtils {

    private Context mContext;

    public FilePickerUtils(Context context) {
        mContext = context;
    }

    /**
     * Process File according to Scoped Storage
     *
     * @param data
     */
    public String processScopedStorageLogic(Intent data) {

        String filename = "";
        try {
            Uri uri = data.getData();
            String mimeType = mContext.getContentResolver().getType(uri);
            if (mimeType == null) {
                String path = getPath(uri);
                if (path == null) {
                    filename = getNameFromURI(uri);
                } else {
                    File file = new File(path);
                    filename = file.getName();
                }
            } else {
                Uri returnUri = data.getData();
                Cursor returnCursor = mContext.getContentResolver().query(returnUri, null, null, null, null);
                int nameIndex = returnCursor.getColumnIndex(OpenableColumns.DISPLAY_NAME);
                returnCursor.moveToFirst();
                filename = returnCursor.getString(nameIndex);
            }
            String sourcePath = mContext.getFilesDir().toString();
            File PROFILE_IMAGE = new File(sourcePath + "/" + "PROFILE_IMAGE");
            if (!PROFILE_IMAGE.exists()) {
                PROFILE_IMAGE.mkdirs();
            }
            //ScopedStorage.getInstance(MyAccount.this).createFolders(ScopedStorage.PROFILE);
            try {
                boolean success = copyFileStream(new File(sourcePath + "/" + "PROFILE_IMAGE" + "/" + filename), uri);
                if (success) {
                    File mFile = new File(sourcePath + "/" + "PROFILE_IMAGE" + "/" + filename);
                    String filePathCopied = mFile.getAbsolutePath();
                    if (isValidFile(filePathCopied)) {
                        if (isValidFileSize(filePathCopied)) {
                            //String userImagepath = mFile.getAbsolutePath();
                            /*String imageType = getImageType(userImagepath);
                            myBitmap = Utils.resizeAndGetCameraBitmap(MyAccount.this, 1000, userImagepath, imageType);
                            imgProfilePic.setImageBitmap(myBitmap);
                            btnSave.setVisibility(View.VISIBLE);*/
                            return mFile.getAbsolutePath();
                        } else {
                            //Toast.makeText(this, getString(R.string.invalidsize), Toast.LENGTH_SHORT).show();
                            return "invalid file picked";
                        }
                    } else {
                        //Toast.makeText(this, getString(R.string.invalidImage), Toast.LENGTH_SHORT).show();
                        return "invalid file picked";
                    }

                } else {
                    return "file not able to copy";
                }
            } catch (Exception e) {
                e.printStackTrace();
                return e.getMessage();
            }

        } catch (Exception e) {
            e.printStackTrace();
            return e.getMessage();
        }
    }

    private boolean copyFileStream(File dest, Uri uri)
            throws IOException {
        InputStream is = null;
        OutputStream os = null;
        try {
            is = mContext.getContentResolver().openInputStream(uri);
            os = new FileOutputStream(dest);
            byte[] buffer = new byte[1024];
            int length;

            while ((length = is.read(buffer)) > 0) {
                os.write(buffer, 0, length);

            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        } finally {
            is.close();
            os.close();
        }

        return true;
    }

    /**
     * Check if Valid File
     * Only JPG,JPEG,PNG should be allowed
     *
     * @param userImagepath
     * @return
     */
    private boolean isValidFile(String userImagepath) {
        boolean isValid;
        String extension = userImagepath.substring(userImagepath.lastIndexOf("."));
        //LogHelper.printLog("TAG_FILE", "" + extension);
        switch (extension) {
            case ".png":
            case ".PNG":
            case ".JPG":
            case ".jpg":
            case ".jpeg":
            case ".JPEG":
                isValid = true;
                break;
            default:
                isValid = false;
                break;
        }
        return isValid;
    }

    /**
     * Get Valid File Size
     *
     * @param userImagepath
     * @return
     */
    private boolean isValidFileSize(String userImagepath) {
        boolean isValid = false;
        File file = new File(userImagepath);
        // Get length of file in bytes
        long fileSizeInBytes = file.length();
        // Convert the bytes to Kilobytes (1 KB = 1024 Bytes)
        long fileSizeInKB = fileSizeInBytes / 1024;
        // Convert the KB to MegaBytes (1 MB = 1024 KBytes)
        long fileSizeInMB = fileSizeInKB / 1024;
        if (fileSizeInMB <= 8) {
            isValid = true;
        }
        return isValid;
    }

    public String getNameFromURI(Uri uri) {
        Cursor c = mContext.getContentResolver().query(uri, null, null, null, null);
        c.moveToFirst();
        return c.getString(c.getColumnIndex(OpenableColumns.DISPLAY_NAME));
    }

    public String getPath(Uri uri) {

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            // DocumentProvider
            if (DocumentsContract.isDocumentUri(mContext, uri)) {
                // ExternalStorageProvider
                if (isExternalStorageDocument(uri)) {
                    final String docId = DocumentsContract.getDocumentId(uri);
                    final String[] split = docId.split(":");
                    final String type = split[0];

                    if ("primary".equalsIgnoreCase(type)) {
                        return Environment.getExternalStorageDirectory() + "/" + split[1];
                    }
                    // TODO handle non-primary volumes
                }
                // DownloadsProvider
                else if (isDownloadsDocument(uri)) {
                    final String id = DocumentsContract.getDocumentId(uri);
                    final Uri contentUri = ContentUris.withAppendedId(Uri.parse("content://downloads/public_downloads"), Long.valueOf(id));
                    return getDataColumn(mContext, contentUri, null, null);
                }
                // MediaProvider
                else if (isMediaDocument(uri)) {
                    final String docId = DocumentsContract.getDocumentId(uri);
                    final String[] split = docId.split(":");
                    final String type = split[0];
                    Uri contentUri = null;
                    if ("image".equals(type)) {
                        contentUri = MediaStore.Images.Media.EXTERNAL_CONTENT_URI;
                    } else if ("video".equals(type)) {
                        contentUri = MediaStore.Video.Media.EXTERNAL_CONTENT_URI;
                    } else if ("audio".equals(type)) {
                        contentUri = MediaStore.Audio.Media.EXTERNAL_CONTENT_URI;
                    }
                    final String selection = "_id=?";
                    final String[] selectionArgs = new String[]{split[1]};
                    return getDataColumn(mContext, contentUri, selection, selectionArgs);
                }
            }
        }
        // MediaStore (and general)
        else if ("content".equalsIgnoreCase(uri.getScheme())) {
            // Return the remote address
            if (isGooglePhotosUri(uri))
                return uri.getLastPathSegment();
            return getDataColumn(mContext, uri, null, null);
        }
        // File
        else if ("file".equalsIgnoreCase(uri.getScheme())) {
            return uri.getPath();
        }
        return null;
    }

    private String getDataColumn(Context context, Uri uri, String selection, String[] selectionArgs) {
        Cursor cursor = null;
        final String column = "_data";
        final String[] projection = {column};
        try {
            cursor = context.getContentResolver().query(uri, projection, selection, selectionArgs, null);
            if (cursor != null && cursor.moveToFirst()) {
                final int index = cursor.getColumnIndexOrThrow(column);
                return cursor.getString(index);
            }
        } finally {
            if (cursor != null)
                cursor.close();
        }
        return null;
    }

    private static boolean isExternalStorageDocument(Uri uri) {
        return "com.android.externalstorage.documents".equals(uri.getAuthority());
    }

    /**
     * @param uri The Uri to check.
     * @return Whether the Uri authority is DownloadsProvider.
     */
    private static boolean isDownloadsDocument(Uri uri) {
        return "com.android.providers.downloads.documents".equals(uri.getAuthority());
    }

    /**
     * @param uri The Uri to check.
     * @return Whether the Uri authority is MediaProvider.
     */
    private static boolean isMediaDocument(Uri uri) {
        return "com.android.providers.media.documents".equals(uri.getAuthority());
    }

    /**
     * @param uri The Uri to check.
     * @return Whether the Uri authority is Google Photos.
     */
    private static boolean isGooglePhotosUri(Uri uri) {
        return "com.google.android.apps.photos.content".equals(uri.getAuthority());
    }
}
